from pylab import *
rc('font', size=12)

def load_data(fname):
    return loadtxt(fname, dtype={
        'names'   : ('size', 'omp', 'mpi', 'setup', 'solve', 'iters'),
        'formats' : ('i8', 'i4', 'i4', 'f8', 'f8', 'i4')
        })

def set_ticks(ax, t):
    ax.set_xticks(t)
    ax.get_xaxis().set_major_formatter(matplotlib.ticker.ScalarFormatter())
    ax.get_xaxis().set_tick_params(which='minor', size=0)
    ax.get_xaxis().set_tick_params(which='minor', width=0)
    setp(ax.get_xticklabels(), fontsize=10, rotation=30)
    setp(ax.get_yticklabels(), fontsize=10)

fig,gs = subplots(4, 2, figsize=(8,10.5))
handles = []

for k,fname in enumerate(('dmem_data/mn4/linear_weak.dat', 'dmem_data/mn4/const_weak.dat')):
    data = load_data(fname)
    for omp in sorted(unique(data['omp'])):
        if omp == 48: continue

        d = data[data['omp'] == omp]
        c = unique(d['mpi'] * omp)
        m = unique(d['mpi'])

        setup = array([min(d[d['mpi']==i]['setup']) for i in m])
        solve = array([min(d[d['mpi']==i]['solve']) for i in m])
        iters = array([min(d[d['mpi']==i]['iters']) for i in m])
        total = setup + solve

        ax = gs[0,k]
        h = ax.loglog(c, total, '.-')
        ax.set_ylim([1e1, 2e2])
        if k == 0: handles.append(h[0])

        ax = gs[1,k]
        ax.loglog(c, setup, '.-')
        ax.set_ylim([1e0, 100])

        ax = gs[2,k]
        ax.loglog(c, solve, '.-')
        ax.set_ylim([5e0, 2e2])

        ax = gs[3,k]
        ax.semilogx(c, iters, '.-')
        ax.set_ylim([0, 400])

    ax = gs[3,k]
    ax.set_xlabel('Number of cores (MPI * OMP)')

for i in range(4):
    for j in range(2):
        set_ticks(gs[i,j], [48 * 2**i for i in range(8)] + [19200])

for fname in ('dmem_data/mn4/trilinos_weak.dat', 'dmem_data/mn4/trilinos_weak_ddml.dat'):
    tri = loadtxt(fname, dtype={
        'names'   : ('mpi', 'size', 'iters', 'setup', 'solve'),
        'formats' : ('i4', 'i8', 'i4', 'f8', 'f8')
        })

    handles += gs[0,0].plot(tri['mpi'], tri['setup'] + tri['solve'], '.-')

    gs[1,0].plot(tri['mpi'], tri['setup'], '.-')
    gs[2,0].plot(tri['mpi'], tri['solve'], '.-')
    gs[3,0].plot(tri['mpi'], tri['iters'], '.-')

gs[0,0].set_title('Linear deflation')
gs[0,0].set_ylabel('Total time')
gs[0,1].set_title('Constant deflation')
gs[1,0].set_ylabel('Setup time')
gs[2,0].set_ylabel('Solve time')
gs[3,0].set_ylabel('Iterations')

tight_layout()

figlegend(handles,
       ['OMP={}'.format(i) for i in (1, 4, 12, 24)] +
       ['Trilinos/ML', 'Trilinos/DD-ML'],
       ncol=3, loc='lower center')
fig.suptitle('Weak scaling of the Poisson problem on the MareNostrum 4 cluster')
fig.subplots_adjust(top=0.93, bottom=0.15)

show()